@@ -6,11 +6,11 @@ import pandas as pd
 
 import scipy.stats
 import scipy.signal
+import scipy.ndimage
 import scipy.cluster.hierarchy
 from scipy.interpolate import UnivariateSpline
 
 import matplotlib
-matplotlib.use('Agg')
 import matplotlib.pyplot as plt
 import matplotlib.patheffects as path_effects
 from matplotlib import cm
@@ -18,16 +18,34 @@ from matplotlib import cm
 import plotly.graph_objects
 from adjustText import adjust_text
 
+from .GenericFunctions import read, write
+
 class VisualizationFunctions:
 
     '''Class of visualization functions for DigitalCellSorter'''
 
-    def __init__(self, dataName = 'dataName', saveDir = os.path.join('')):
+    def __init__(self, dataName = 'dataName', saveDir = os.path.join(''), matplotlibMode = 'Agg'):
 
         '''Function called automatically'''
 
         self.saveDir = saveDir
         self.dataName = dataName
+        self.matplotlibMode = matplotlibMode
+
+        return
+
+    @property
+    def matplotlibMode(self):
+
+        return self._matplotlibMode
+
+    @matplotlibMode.setter
+    def matplotlibMode(self, value):
+
+        self._matplotlibMode = value
+
+        if not self.matplotlibMode is None:
+            matplotlib.use(self.matplotlibMode)
 
         return
 
@@ -64,27 +82,32 @@ class VisualizationFunctions:
         try:
             if not extension[0] == '.':
                 extension = ''.join(['.', extension])
-        except:
+        except Exception as exception:
+            print(exception)
             print('Figure extension/format error')
             print('Example of acceptable extension: \".png\"')
 
             return
 
         if extension in ['.png', '.jpeg', '.tiff']:
+            try:
+                fig.savefig(os.path.join(saveDir, label + extension), dpi=dpi)
+            except Exception as exception:
+                print(exception)
 
-            fig.savefig(os.path.join(saveDir, label + extension), dpi=dpi)
-    
         elif extension in ['.svg', '.eps', '.pdf']:
-
-            fig.savefig(os.path.join(saveDir, label + extension))
-
+            try:
+                fig.savefig(os.path.join(saveDir, label + extension))
+            except Exception as exception:
+                print(exception)
         else:
             print('Unsupported format. Figure not saved')
 
         if close:
             try:
                 plt.close(fig)
-            except:
+            except Exception as exception:
+                print(exception)
                 print('Error while closing figure')
 
         return
@@ -184,7 +207,7 @@ class VisualizationFunctions:
 
         return
     
-    def makeMarkerExpressionPlot(self, dpi = 600, extension = 'png'):
+    def makeMarkerExpressionPlot(self, dpi = 300, extension = 'png'):
 
         '''Produce image on marker genes and their expression on all clusters.
         Uses files generated by function DCS.Vote
@@ -353,7 +376,7 @@ class VisualizationFunctions:
 
         return
 
-    def internalMakeMarkerSubplots(self, df, X_projection, hugo_cd_dict, NoFrameOnFigures = False, HideClusterLabels = False, outlineClusters = True, analyzeBy = 'cluster'):
+    def internalMakeMarkerSubplots(self, df, X_projection, hugo_cd_dict, NoFrameOnFigures = False, HideClusterLabels = False, outlineClusters = True, analyzeBy = 'cluster', saveSubDir = 'marker_subplots', dpi = 300, extension = 'png'):
 
         '''Produce subplots on each marker and its expression on all clusters
 
@@ -391,7 +414,7 @@ class VisualizationFunctions:
             DCS.internalMakeMarkerSubplots(df_markers_expr, projection, hugo_cd_dict)
         '''
         
-        def MarkerSubplot(counter, marker, df, votingResults, X_projection, cellClusterIndexLabel, hugo_cd_dict, dataName, saveDir, NoFrameOnFigures, HideClusterLabels, XLIM, YLIM, directory, circles):
+        def MarkerSubplot(counter, marker, df, analyzeBy, X_projection, cellClusterIndexLabel, hugo_cd_dict, dataName, saveDir, NoFrameOnFigures, HideClusterLabels, XLIM, YLIM, directory, circles):
 
             fig,ax = plt.subplots(figsize=(8,8))
 
@@ -422,7 +445,7 @@ class VisualizationFunctions:
                 x_mean = np.mean(X_projection2_cluster[0,:])
                 y_mean = np.mean(X_projection2_cluster[1,:])
 
-                _text_label = votingResults[label] if not HideClusterLabels else ''
+                _text_label = label if not HideClusterLabels else ''
 
                 ax.text(x_mean,y_mean,
                         _text_label.
@@ -447,14 +470,9 @@ class VisualizationFunctions:
                 ax.axis('off')
             fig.tight_layout()
             if self.saveDir is not None: 
-                #fig.savefig('%s/marker_subplots/%s_%s_%s.png' %
-                #(self.saveDir,self.dataName,marker,suffix.replace(',','_').replace('/','_')),dpi=150)
-                #fig.savefig(os.path.join(self.saveDir, 'marker_subplots', '%s_%s_%s.pdf' %
-                #(self.dataName,marker,suffix.replace(',','_').replace('/','_'))))
-                fig.savefig(os.path.join(self.saveDir, 'marker_subplots', '%s_%s_%s.png' % (self.dataName,marker,suffix.replace(',','_').replace('/','_'))), dpi=300)
-                print(marker, end=" ", flush=True)
+                self.saveFigure(fig, directory, '%s_%s_%s_%s' % (self.dataName,marker,suffix.replace(',','_').replace('/','_'),analyzeBy), extension=extension, dpi=dpi)
 
-            plt.close(fig)
+                print(marker, end=" ", flush=True)
 
             return
         
@@ -465,30 +483,26 @@ class VisualizationFunctions:
         XLIM = [mins[0] - deltas[0],maxs[0] + deltas[0]]
         YLIM = [mins[1] - deltas[1],maxs[1] + deltas[1]]
 
-        directory = os.path.join(self.saveDir, 'marker_subplots', '')
-        if not os.path.exists(directory):
-            os.makedirs(directory)
-
         if len(df.index) > 1:
             print('\nSaving marker expression plots:\n')
         else:
             print('Saving expression plot of:', end=' ', flush=True)
 
-        votingAnalyzeBy = 'Predicted cell type' if analyzeBy == 'label' else analyzeBy
-
-        df_votingResults = pd.read_excel(os.path.join(self.saveDir, self.dataName + '_annotation.xlsx'), sheet_name='z-scores')
-        votingResults = dict(zip(df_votingResults[votingAnalyzeBy].values, df_votingResults['Predicted cell type'].values))
-
-        cellClusterIndexLabel = df.columns.get_level_values(analyzeBy).values
+        if analyzeBy == 'celltype':
+            try:
+                index = df.columns.get_level_values('label').str.split(' #', expand=True).get_level_values(0).values
+            except:
+                index = df.columns.get_level_values('celltype').values
+        else:
+            index = df.columns.get_level_values(analyzeBy).values
 
         for counter,marker in enumerate(df.index.values):
-            MarkerSubplot(counter, marker, pd.DataFrame(data=np.reshape(np.array(df.loc[marker]), (1,len(df.loc[marker]))), columns=df.columns, index=[marker]), 
-                            votingResults, X_projection, cellClusterIndexLabel, hugo_cd_dict, self.dataName, self.saveDir, NoFrameOnFigures, HideClusterLabels, XLIM, YLIM, directory, outlineClusters)
+            MarkerSubplot(counter, marker, pd.DataFrame(data=np.reshape(np.array(df.loc[marker]), (1,len(df.loc[marker]))), columns=df.columns, index=[marker]), analyzeBy, X_projection, index, hugo_cd_dict, self.dataName, self.saveDir, NoFrameOnFigures, HideClusterLabels, XLIM, YLIM, os.path.join(self.saveDir, saveSubDir, ''), outlineClusters)
         print()
 
         return
 
-    def makeAnnotationResultsMatrixPlot(self, dpi = 600, extension = 'png'):
+    def makeAnnotationResultsMatrixPlot(self, dpi = 300, extension = 'png'):
 
         '''Produce voting results voting matrix plot
 
@@ -540,9 +554,6 @@ class VisualizationFunctions:
         ax = fig.add_axes([0.15, 0.125, 0.65, 0.85])
         axx = fig.add_axes([0.76, 0.125, 0.19, 0.85])
 
-        #ax = plt.subplot(gs[0])
-        #axx = plt.subplot(gs[1])
-
         zscores = df_votingResults[cellTypes].values
         ax.imshow(zscores, aspect=1, cmap='Greens', vmin=0, vmax=0.5, interpolation='None')
 
@@ -587,10 +598,7 @@ class VisualizationFunctions:
         axx.set_xlabel('Number of\ncells in clusters', fontsize=20)
         axx.set_ylim(-0.5, num_of_clusters - 0.5)
         
-        #fig.tight_layout()
-        fig.savefig(os.path.join(self.saveDir, self.dataName + '_scores_matrix.png'), dpi=150)
-
-        plt.close(fig)
+        self.saveFigure(fig, self.saveDir, self.dataName + '_scores_matrix', extension=extension, dpi=dpi)
  
         return 
 
@@ -616,7 +624,8 @@ class VisualizationFunctions:
 
         try:
             df_noise_dict = pd.read_excel(os.path.join(self.saveDir, self.dataName + '_annotation.xlsx'), sheet_name='Null distributions', index_col=0, header=[0,1], skiprows=[2])
-        except:
+        except Exception as exception:
+            print(exception)
             print('Error loading distributions from the results file')
 
             return
@@ -660,7 +669,8 @@ class VisualizationFunctions:
                 minx, maxx = np.round(df_noise_dict.index.values[np.where(df_noise_dict.xs(key=cell_types[i], level=0, axis=1).values.sum(axis=1) != 0.)[0][[0,-1]]], 3)
                 #maxx += 0.1
                 maxx = np.round(maxx, 3)
-            except:
+            except Exception as exception:
+                print(exception)
                 minx = np.round(df_noise_dict.index.values[0], 3)
                 maxx = np.round(df_noise_dict.index.values[-1], 3)
 
@@ -716,15 +726,13 @@ class VisualizationFunctions:
 
             print()
         
-        fig.savefig(os.path.join(self.saveDir, self.dataName + '_null_distributions.png'), dpi=1200)
-
-        plt.close(fig)
+        self.saveFigure(fig, self.saveDir, self.dataName + '_null_distributions', extension=extension, dpi=dpi)
 
         matplotlib.rcParams['axes.linewidth'] = origWidth
 
         return
 
-    def makeProjectionPlot(self, Xprojection, cellClusterIndexLabel, suffix, colormap = cm.jet, legend = True, labels = True, colorbar = False, fontsize = 10, plotNaNs = True, rightShift = 0.3, dpi = 600, extension = 'png'):
+    def makeProjectionPlot(self, Xprojection, cellClusterIndexLabel, suffix = '', colormap = cm.jet, legend = True, labels = True, colorbar = False, fontsize = 10, plotNaNs = True, rightShift = 0.3, dpi = 300, extension = 'png'):
 
         '''Produce projection plot (2D layout) with a specified coloring scheme
 
@@ -809,7 +817,13 @@ class VisualizationFunctions:
         texts = []
 
         for ilabel, label in enumerate(possible_cluster_labels):
-            color = colormap(ilabel / len(possible_cluster_labels)) if type(colormap) is matplotlib.colors.LinearSegmentedColormap else colormap[label.split(' #')[0]]
+
+            if type(colormap) is matplotlib.colors.LinearSegmentedColormap:
+                color = colormap(ilabel / len(possible_cluster_labels))
+            elif type(colormap) is str:
+                color = plt.get_cmap(colormap)(ilabel / len(possible_cluster_labels))
+            else:
+                color = colormap[label.split(' #')[0]]
 
             XprojectionC = Xprojection[:,cellClusterIndexLabel == label]
 
@@ -840,14 +854,11 @@ class VisualizationFunctions:
         if colorbar:
             add_colorbar(fig, possible_cluster_labels, cmap=colormap, fontsize=fontsize)
 
-        if self.saveDir is not None: 
-            fig.savefig(os.path.join(self.saveDir, '%s_clusters_%s.png' % (self.dataName, suffix)), dpi=300)
-
-        plt.close(fig)
+        self.saveFigure(fig, self.saveDir, '%s_clusters_%s' % (self.dataName, suffix), extension=extension, dpi=dpi)
 
         return
 
-    def makeStackedBarplot(self, clusterName = None, legendStyle = False, includeLowQC = True, dpi = 600, extension = 'png'):
+    def makeStackedBarplot(self, clusterName = None, legendStyle = False, includeLowQC = True, dpi = 300, extension = 'png'):
         
         '''Produce stacked barplot with cell fractions
 
@@ -916,7 +927,8 @@ class VisualizationFunctions:
                     df_main.loc[self.nameForLowQC, df_main.columns[1]] = -1
 
                     colors[self.nameForLowQC] = (0.6, 0.6, 0.6, 1.)
-                except:
+                except Exception as exception:
+                    print(exception)
                     print('QC data not found')
 
             df_main = df_main.apply(lambda x: 100. * x / np.sum(df_main, axis=0), axis=1).loc[np.sum(df_main, axis=1) > 0].sort_values(by=[s]).drop(columns=[s])
@@ -928,8 +940,6 @@ class VisualizationFunctions:
 
         df_Main, colors, clusterName = get_stacked_data_and_colors(self.saveDir)
 
-        saveName = os.path.join(self.saveDir, "%s_subclustering_stacked_barplot_%s.png" % (self.dataName, ('All cell clusters' if clusterName == None else clusterName).replace(' ', '_').replace('*', '')))
-
         if legendStyle:
             fig,ax = plt.subplots(figsize=(4.5,8)) #4.15
         else:
@@ -981,12 +991,8 @@ class VisualizationFunctions:
         if legendStyle:
             fig.tight_layout()
 
-        try:
-            fig.savefig(saveName, dpi=300)
-
-            plt.close(fig)
-        except:
-            return
+        saveName = "%s_subclustering_stacked_barplot_%s" % (self.dataName, ('All cell clusters' if clusterName == None else clusterName).replace(' ', '_').replace('*', ''))
+        self.saveFigure(fig, self.saveDir, saveName, extension=extension, dpi=dpi)
 
         print('Saved stacked bar plot: %s' % ('All cell clusters' if clusterName == None else clusterName))
 
@@ -1063,7 +1069,8 @@ class VisualizationFunctions:
 
         try:
             title = os.path.basename(plotPathAndName)
-        except:
+        except Exception as exception:
+            print(exception)
             title = plotPathAndName
 
         if includeTitle:
@@ -1084,7 +1091,8 @@ class VisualizationFunctions:
 
         try:
             x, y = cutoff, sg[np.where(spline_data.T[0] >= cutoff)[0][0]]
-        except:
+        except Exception as exception:
+            print(exception)
             x, y = cutoff, 0.
 
         ax.plot([x,x], [0,y], 'k', lw=2)
@@ -1189,7 +1197,8 @@ class VisualizationFunctions:
             temp_columns = pd.MultiIndex.from_arrays([df.columns, [colormapForColumns[item] for item in df.columns]], names=['label', 'color'])
             df.index = temp_index
             df.columns = temp_columns
-        except:
+        except Exception as exception:
+            print(exception)
             print('Using default node colors')
             colormapForIndex = None
             colormapForColumns = None
@@ -1216,21 +1225,25 @@ class VisualizationFunctions:
                     newColor = ','.join(nodeColors[sources[i]].split(',')[:3] + ['0.6)'])
                     colorscales[i] = dict(label=labels[i], colorscale=[[0, newColor], [1, newColor]])
 
-        fig = plotly.graph_objects.Figure(data=[plotly.graph_objects.Sankey(valueformat = '', valuesuffix = '',
-            node = dict(pad = 20, thickness = 40, line = dict(color = 'white', width = 0.5), label = nodeLabels, color = nodeColors,),
-            link = dict(source = sources, target = targets, value = values, label = labels, colorscales = colorscales))]) #line ={'color':'rgba(255,0,0,0.8)', 'width':0.1}
+        try:
+            fig = plotly.graph_objects.Figure(data=[plotly.graph_objects.Sankey(valueformat = '', valuesuffix = '',
+                node = dict(pad = 20, thickness = 40, line = dict(color = 'white', width = 0.5), label = nodeLabels, color = nodeColors,),
+                link = dict(source = sources, target = targets, value = values, label = labels, colorscales = colorscales))]) #line ={'color':'rgba(255,0,0,0.8)', 'width':0.1}
 
-        if not title is None:
-            fig.update_layout(title_text=title, font_size=10)
+            if not title is None:
+                fig.update_layout(title_text=title, font_size=10)
 
-        fig.write_image(os.path.join(self.saveDir, self.dataName + nameAppend + '.png'), width=500, height=500, scale=quality)
+            fig.write_image(os.path.join(self.saveDir, self.dataName + nameAppend + '.png'), width=500, height=500, scale=quality)
+
+        except Exception as exception:
+            print(exception)
 
         if interactive:
             fig.show()
 
         return
 
-    def makePlotOfNewMarkers(self, df_marker_cell_type, df_new_marker_cell_type, dpi = 600, extension = 'png'):
+    def makePlotOfNewMarkers(self, df_marker_cell_type, df_new_marker_cell_type, dpi = 300, extension = 'png'):
 
         '''Produce plot of the new markers extracted from the annotated clusters
 
@@ -1287,13 +1300,11 @@ class VisualizationFunctions:
                     ax.plot(xy.T[0], xy.T[1], 'ro', markeredgecolor='r', ms=1.0, markeredgewidth=0.2)
 
         #ax.set_title('Additional markers along with the overlapping part of the input (red)')
-        fig.savefig(os.path.join(self.saveDir, self.dataName + '_new_markers.png'), dpi=600)
-
-        plt.close(fig)
+        self.saveFigure(fig, self.saveDir, self.dataName + '_new_markers', extension=extension, dpi=dpi)
 
         return None
 
-    def makeTtestPlot(self, df, dfp, label = None, reorder = True, p_value_cutoff = 0.05, dpi = 600, extension = 'png'):
+    def makeTtestPlot(self, df, dfp, label = None, reorder = True, p_value_cutoff = 0.05, dpi = 300, extension = 'png'):
 
         '''Produce heatmap plot of t-test p-Values calculated gene-pair-wise
         from the annotated clusters.
@@ -1389,14 +1400,11 @@ class VisualizationFunctions:
 
         axisColor.set_yticklabels([np.round(dataMax,2), np.round(dataMin,2)])
 
-        fig.savefig(os.path.join(self.saveDir, self.dataName + '_ttest_%s.png' % (label.replace('\n', '_'))), dpi=300)
-
-        plt.close(fig)
+        self.saveFigure(fig, self.saveDir, self.dataName + '_ttest_%s' % (label.replace('\n', '_')), extension=extension, dpi=dpi)
 
         return None
     
-    def makeCellMarkersPiePlot(self, type1, type2, df_marker_cell_type = 'all', nameToAppend = None, 
-                               listUnexpressedMarkers = True, orthogonalSectorsShift = 0.1, rotationAngle = 0, dpi = 600, extension = 'png'):
+    def makeCellMarkersPiePlot(self, type1, type2, df_marker_cell_type = 'all', nameToAppend = None, listUnexpressedMarkers = True, orthogonalSectorsShift = 0.1, rotationAngle = 0, dpi = 300, extension = 'png'):
 
         '''Make summary of markers comparison between two cell types.
 
@@ -1444,7 +1452,8 @@ class VisualizationFunctions:
         try:
             df_marker_expression = pd.read_excel(os.path.join(self.saveDir, self.dataName + '_annotation.xlsx'), 
                                                     sheet_name='Marker cell type weight matrix', index_col=0, header=0).T
-        except:
+        except Exception as exception:
+            print(exception)
             print('Marker expression data unavailable')
             df_marker_expression = None
             listUnexpressedMarkers = False
@@ -1480,7 +1489,8 @@ class VisualizationFunctions:
             try:
                 pos = set(df.index[(df.loc[:, celltype] > 0.)].values)
                 neg = set(df.index[(df.loc[:, celltype] < 0.)].values)
-            except:
+            except Exception as exception:
+                print(exception)
                 print('Cell type %s not found' % (celltype))
                 print('Available celltypes are: %s' % (df.columns.values.tolist()))
 
@@ -1518,7 +1528,8 @@ class VisualizationFunctions:
                 t2pe, t2ne = getSet(df_marker_expression, type2)
                 
                 setsE, allE = getEightAll(t1pe, t2pe, t1ne, t2ne)
-            except:
+            except Exception as exception:
+                print(exception)
                 listUnexpressedMarkers = False
 
         labels = '+/*', '+/-', '*/-', '-/-', '-/*', '-/+', '*/+', '+/+'
@@ -1658,12 +1669,304 @@ class VisualizationFunctions:
         ax.text(0., 0., '%s\n(%s)' % (len(all), len(all.difference(allE))) if listUnexpressedMarkers else '%s' % (len(all)), 
                 color='k', fontsize=8, ha='center', va='center').set_path_effects([path_effects.Stroke(linewidth=1, foreground='blue'),path_effects.Normal()])
 
-        if self.saveDir is not None:
-            fig.savefig(os.path.join(self.saveDir, self.dataName + 'Markers_of_%s_vs_%s_(%s)_%s.png' % (type1.replace('/',''), 
-                                                                                                  type2.replace('/',''), 
-                                                                                                  nameToAppend, 
-                                                                                                  additional_name)), dpi=300)
+        saveName = 'Markers_of_%s_vs_%s_(%s)_%s' % (type1.replace('/',''), type2.replace('/',''), nameToAppend, additional_name)
+        self.saveFigure(fig, self.saveDir, self.dataName + saveName, extension=extension, dpi=dpi)
 
-            plt.close(fig)
+        return dict(zip(labels, list(sets)))
 
-        return dict(zip(labels, list(sets)))
\ No newline at end of file
+    def makeHopfieldPCplot(self, colormap = cm.hot_r, plotTrLines = False, clusterid = 1, trID = 0, axisOff = False, trPath = None, dpi = 300, extension = 'png'):
+
+        '''Make radar plot of the attractors in their principal components coordinates
+
+        Parameters:
+            colormap: matplotlib.colormap or str, Default cm.hot_r
+                Colormap or its string name
+
+            plotTrLines: boolean, Default False
+                Whether to plot trajectories
+
+            clusterid: int, Default 1
+                Identifier of the cluster to plot trajectories of
+
+            trID: int, Default 0
+                Identifier of the trajectories to plot
+
+            axisOff: boolean, Default False
+                Whether to hide the axes lines
+
+            trPath: str, Default None
+                Path to trajectories files
+
+            dpi: int, Default 300
+                Resolution of the figure
+
+            extension: str, Default 'png'
+                Format extension of the figure
+
+        Returns:
+            None
+        
+        Usage:
+            DCS = DigitalCellSorter.DigitalCellSorter()
+
+            DCS.makeHopfieldPCplot()
+        '''
+
+        if axisOff:
+            ax.axis('off')
+
+        if trPath is None:
+            trPath = os.path.join(self.saveDir, 'HopfieldTrajectories')
+
+        if not os.path.exists(trPath):
+            print('Data not found', flush=True)
+
+            return
+
+        fig = plt.figure(figsize=(8,8))
+        ax = plt.subplot(111, polar=True, theta_direction=-1, theta_offset=0.5*np.pi)
+
+        attrs, attrs_names = read(os.path.join(trPath, 'attrs'))
+        N = attrs.shape[1]
+        df = pd.DataFrame(data=attrs[:N], index=attrs_names, 
+                          columns=['PC%s\n%s%%'%(i+1, np.int(100.*attrs[N][i])) for i in range(N)])
+
+        wherePC = attrs[N] > 0.001
+        df = df[df.columns[wherePC]]
+        N = df.shape[1]
+
+        vmaxAt = df.max(axis=0).max()
+        vminAt = df.min(axis=0).min()
+
+        ax.set_ylim(vminAt, vmaxAt)
+ 
+        angles = [n / float(N) * 2 * np.pi for n in range(N)] + [0.]
+ 
+        ax.set_xticklabels([])
+        ax.set_xticks(angles)
+
+        for i, celltype in enumerate(df.index):
+            values=df.loc[celltype].values.flatten().tolist()
+            values.append(values[0])
+
+            color = cm.jet(i/len(attrs_names))
+
+            ax.plot(angles, values, color=color, linewidth=1.0, linestyle='solid', label=celltype)
+            #ax.fill(angles, values, alpha=0.2, color=color)
+
+            temp_texts = ax.text(angles[np.argmax(values)], values[np.argmax(values)], celltype, color=color, fontsize=10, ha='center', va='center')
+            temp_texts.set_path_effects([path_effects.Stroke(linewidth=0.5, foreground='k'), path_effects.Normal()])
+
+        if plotTrLines:
+            trajectories = read(os.path.join(trPath, 'trajectories%s')%(trID))
+
+            initial, final, typesNames, clusterNames = read(os.path.join(trPath, 'additional'))
+        
+            thisTr = trajectories[:, clusterid, wherePC].T
+
+            vmax = max(thisTr.max(axis=0).max(axis=0), vmaxAt)
+            vmin = min(thisTr.min(axis=0).min(axis=0), vminAt)
+
+            ax.set_ylim(vmin, vmax)
+
+            inId = initial[clusterid]
+            outId = final[clusterid]
+
+            print('ClusterID: %s, Initial state: %s (%s), Final state: %s (%s)'%(clusterNames[clusterid], typesNames[inId], inId, typesNames[outId], outId))
+
+            suffix = clusterNames[clusterid]
+
+            fig.suptitle('Cluster %s: %s -> %s' % (clusterNames[clusterid], typesNames[inId] if inId!=-1 else 'Unknown', typesNames[outId] if outId!=-1 else 'Unknown'), fontsize=11, color='b')
+
+            values = thisTr.T[0].tolist() + [thisTr.T[0][0]]
+
+            timeLimit = thisTr.shape[1]
+
+            for t in range(timeLimit):
+                values = thisTr.T[t].tolist() + [thisTr.T[t][0]]
+
+                if t==0:
+                    ax.plot(angles, values, 'o-', ms=4.0, color='b', alpha=1.0, clip_on=False)
+
+                ax.plot(angles, values, color='b', linewidth=0.5, alpha=0.04, linestyle='solid')
+                ax.fill(angles, values, alpha=0.005, color='b')
+
+                if t == timeLimit - 1:
+                    ax.plot(angles, values, 'o-', ms=4.0, color='r', alpha=1.0, clip_on=False)
+
+            ax.set_rlabel_position(0)
+        else:
+            vmax = vmaxAt
+            vmin = vminAt
+            suffix = 'attractors'
+
+        for i, pc in enumerate(df.columns):
+            temp_texts = ax.text(angles[i], 1.1 * (vmax - vmin) + vmin, pc, color='k', fontsize=14, ha='center', va='center')
+            temp_texts.set_path_effects([path_effects.Stroke(linewidth=0.5, foreground='w'), path_effects.Normal()])
+
+        self.saveFigure(fig, self.saveDir, self.dataName + '_polar_%s'%(suffix), extension=extension, dpi=dpi)
+
+        return
+
+    def makeHopfieldLandscapePlot(self, legend = False, labels = False, PCx = 0, PCy = 1, colorbar = True, fontsize = 10, plotMesh = True, plotAttractors = True, adjustText = True, axisOff = True, colorbarva = 0.75, colorbarha = 0.85, trPath = None, colormap = matplotlib.colors.LinearSegmentedColormap.from_list('cmap', [(1, 1, 1), (0, 1, 1), (0, 0, 1), (1, 0, 0)], N=1000), dpi = 300, extension = 'png'):
+
+        '''Make heatmap plot of the attractors in their two principal components coordinates
+
+        Parameters:
+            legend: boolean, Default False
+                Whether to add legend containing cell types names
+            
+            labels: boolean, Default False
+                Whether to add labels 
+                
+            PCx: int, Default 0
+                Principal component for x-coordinate of the plot
+                            
+            PCy: int, Default 1
+                Principal component for y-coordinate of the plot
+            
+            colorbar: boolean, Default False
+                Whether to add colorbar
+
+            fontsize: int, Default 10
+                Text labels font size
+            
+            plotMesh: boolean, Default False
+                Whether to plot landscape heatmap
+                
+            plotAttractors: boolean, Default False
+                Whether to plot attractor stars
+                
+            adjustText: boolean, Default False
+                Whether to minimize text labels overlap
+                
+            axisOff: boolean, Default False
+                Whether to hide the axes lines
+
+            colorbarva: float, Default 0.75
+                Vertical position of the bottom of the colorbar
+
+            colorbarha: float, Default 0.85
+                Horizontal position of the colorbar
+
+            trPath: str, Default None
+                Path to trajectories files
+
+            colormap: matplotlib.colormap or str, Default matplotlib.colors.LinearSegmentedColormap.from_list('cmap', [(1, 1, 1), (0, 1, 1), (0, 0, 1), (1, 0, 0)], N=1000)
+                Colormap or its string name
+
+            dpi: int, Default 300
+                Resolution of the figure
+
+            extension: str, Default 'png'
+                Format extension of the figure
+
+        Returns:
+            None
+        
+        Usage:
+            DCS = DigitalCellSorter.DigitalCellSorter()
+
+            DCS.makeHopfieldLandscapePlot()
+        '''
+
+        np.random.seed(0)
+
+        colormap.set_bad('white')
+
+        def add_colorbar(fig, labels, cmap = matplotlib.colors.LinearSegmentedColormap.from_list('GR', [(0, 1, 0), (1, 0, 0)], N=100), fontsize = 10):
+    
+            mapp = cm.ScalarMappable(norm=matplotlib.colors.Normalize(vmin=np.min(labels), vmax=np.max(labels)), cmap=cmap)
+            sp = np.linspace(np.max(labels), np.min(labels), num=6, endpoint=True)
+            mapp.set_array(sp)
+
+            axisColor = fig.add_axes([colorbarha, colorbarva, 0.01, 0.2])
+
+            fig.colorbar(mapp, cax=axisColor, ticks=sp)
+
+            axisColor.tick_params(labelsize=fontsize)
+            axisColor.set_yticklabels(sp.astype(int))
+    
+            return None
+
+        fig = plt.figure(figsize=(8,8))
+        ax = fig.add_axes([0.05,0.05,0.9,0.9])
+
+        if axisOff:
+            ax.axis('off')
+            
+        if trPath is None:
+            trPath = os.path.join(self.saveDir, 'HopfieldTrajectories')
+
+        if not os.path.exists(trPath):
+            print('Data not found', flush=True)
+
+            return
+
+        attrs_xpca, attrs_names = read(os.path.join(trPath, 'attrs'))
+        attrs_xpca = attrs_xpca[:attrs_xpca.shape[1]]
+
+        mesh_xpca = read(os.path.join(trPath, 'mesh'))
+
+        mesh_energy = mesh_xpca[range(len(mesh_xpca)), -1]
+        mesh_xpca = mesh_xpca[range(len(mesh_xpca)), :-1]
+
+        data = np.vstack([attrs_xpca, mesh_xpca])
+
+        coords = data.T[[PCx, PCy], :]
+        attrs2D, mesh2D = coords[:, :attrs_xpca.shape[1]].T, coords[:, attrs_xpca.shape[1]:].T
+    
+        if plotMesh:
+            vmin, vmax = min(mesh_energy), 0.
+
+            vals = mesh_energy.copy()
+            vals[np.where(vals > (vmax - 0.001))[0]] = vmax - 0.001
+            vals[np.where(vals < (vmin + 0.001))[0]] = vmin + 0.001
+
+            xmin, xmax = mesh2D.T[0].min(), mesh2D.T[0].max()
+            ymin, ymax = mesh2D.T[1].min(), mesh2D.T[1].max()
+
+            ngrid = 100
+            grid = np.zeros((ngrid + 1, ngrid + 1))
+            grid[:] = vmin
+
+            i = (ngrid * (mesh2D.T[0] - xmin) / (xmax - xmin)).astype(int)
+            j = (ngrid * (mesh2D.T[1] - ymin) / (ymax - ymin)).astype(int)
+
+            se = pd.Series(index=zip(i,j), data=mesh_energy).groupby(axis=0, level=0).agg(np.min)
+            se.index = pd.MultiIndex.from_tuples(se.index)
+
+            grid[(se.index.get_level_values(0).values, se.index.get_level_values(1).values)] = se.values
+
+            maskedArray = np.ma.array(grid.T, mask=np.isnan(grid.T,))
+
+            im = ax.imshow(maskedArray[::-1], vmin=vmin, vmax=vmax, cmap=colormap, alpha=0.8,
+                        extent=[xmin, xmax, ymin, ymax], interpolation='quadric', zorder=-10**8, clip_on=False)
+
+            data = scipy.ndimage.gaussian_filter(grid.T, 1.5)
+            xgrid = np.linspace(xmin, xmax, num=(ngrid+1))
+            ygrid = np.linspace(ymin, ymax, num=(ngrid+1))
+
+            tempColormap = matplotlib.colors.LinearSegmentedColormap.from_list('cmap', [(0.75, 0.75, 0.75), (0, 1, 1), (0, 0, 1), (1, 0, 0)], N=1000)
+
+            ax.contour(xgrid, ygrid, data, levels=10, cmap=tempColormap, linewidths=0.5, zorder=-10**8+1)
+    
+        if plotAttractors:
+            texts = []
+
+            ax.plot(attrs2D.T[0], attrs2D.T[1], '*', ms=14, color='k', alpha=1.0, zorder=-10**7, clip_on=False)
+            for attr in range(attrs2D.T[0].shape[0]):
+                temp_texts = ax.text(attrs2D.T[0][attr], attrs2D.T[1][attr], attrs_names[attr], fontsize=10, ha='left',va='center', zorder=10 ** 10, clip_on=False)
+                temp_texts.set_path_effects([path_effects.Stroke(linewidth=1, foreground='white'), path_effects.Normal()])
+                texts.append(temp_texts)
+
+            if adjustText:
+                adjust_text(texts, arrowprops=dict(arrowstyle='-', color='k', lw=0.3, alpha=0.5), force_text=(0.05, 0.05))
+
+        if plotMesh and colorbar:
+
+            add_colorbar(fig, [vmax, vmin], cmap=colormap, fontsize=fontsize)
+
+        self.saveFigure(fig, self.saveDir, self.dataName + '_energy_landscape_PC%s_vs_PC%s'%(PCy, PCx), extension=extension, dpi=dpi)
+
+        return
